/* $Id: zonemanip.c,v 1.12 2003/12/28 00:20:13 boll Exp $ 
 *
 * Written by Soeren Boll Overgaard <boll@fork.dk>
 *
 * Released under the GPL version 2, or any later version.
 */
#include "zonemanipfunc.h"

int main(int argc,char *argv[]) {
  char* this_user =                  NULL;
  static char* entry_string =        NULL;
  static char* entry_target_string = NULL;
  char* db_host =                    DBHOST;
  char* db_user =                    DBUSER;
  char* db_name =                    DBNAME;
  char* db_pass =                    DBPASS;
  static char* entry_type =          NULL;
  static char* pdns_conf =           PDNSCONF;
  char* domain_name =                NULL;
  const char** error_msg;
  static int entry_priority =        0;
  static int entry_ttl =             21600;
  static int add_command =           0;
  static int rem_command =           0;
  static int arecord_command =       0;
  static int cname_command =         0;
  static int mx_command =            0;
  static int ns_command =            0;
  static int soa_command =           0;
  static int aaaa_command =          0;
  static int ptr_command =           0;
  static int batch_mode =            0;
  static int list_command =          0;
  int domain_id =                    0;
  struct timeb *tp;
  const char *name, *type, *content;
  int prio, ttl, max_name_length=0, max_content_length=0;

  dbi_conn conn;
  dbi_result result;

  /* Parse arguments */
  optStruct opt[] = {
    { 'l',"list",OPT_FLAG, &list_command, 0 },
    { 'y',"yes",OPT_FLAG, &batch_mode, 0 },
    { 'a',"add", OPT_FLAG, &add_command, 0 },
    { 'r',"remove", OPT_FLAG, &rem_command, 0 },
    { 'A',"a-record", OPT_FLAG, &arecord_command, 0 },
    { 'C',"cname", OPT_FLAG, &cname_command, 0 },
    { 'M',"mx", OPT_FLAG, &mx_command, 0 },
    { 'N',"ns", OPT_FLAG, &ns_command, 0 },
    { 'B',"AAAA",OPT_FLAG, &aaaa_command, 0},
    { 'P',"ptr",OPT_FLAG, &ptr_command, 0},
    { 'c',"pdns-config", OPT_STRING, &pdns_conf, 0},
    { 'e',"entry", OPT_STRING, &entry_string, 0 },
    { 't',"target", OPT_STRING, &entry_target_string, 0 },
    { 'p',"priority",OPT_INT, &entry_priority, 0 },
    { 'v',"version",OPT_FLAG,(void *)version, OPT_CALLFUNC },
    { 'h',"help",OPT_FLAG, (void *)usage, OPT_CALLFUNC },
    { 0, 0, OPT_END, 0, 0 }
  };

  optParseOptions(&argc, argv, opt, 0);

  if (argc!=2 || (strlen(argv[1])<4))
    error("You did not specify a valid domain name");
  
  /* Get the domain name */
  domain_name=argv[1];

  /* Check user credentials */
  this_user = getlogin();

  /* Determine dbuser, dbhost, dbname and dbpass */
  get_pdns_mysql_db_credentials(&pdns_conf,&db_user,&db_pass,&db_name,&db_host);

  /* Do a bit of sanity checking */
  if (list_command) {
    if (add_command || rem_command)
      error("List, add and remove commands can not be combined");
  } 
  else if (add_command || rem_command) {
    if (add_command && rem_command)
      error("Add and remove commands can not be combined");
    if (list_command)
      error("List, add and remove commands can not be combined");
    if (!mx_command && !ns_command && !cname_command && !arecord_command)
      error("You must specify the type of record to manipulate");
    if (!entry_string) 
      error("You must specify the name of the entry");
    if (!entry_target_string)
      error("You must specify a target for the record");
    if (entry_string && !is_valid_domain(entry_string))
      error("The entry you specified is not a valid domain part");
    if (entry_target_string && !is_valid_domain(entry_target_string)) 
      error("The target you specified is not a valid domain part");
    if (entry_priority && ((entry_priority<0) || (entry_priority>0xFFFF)))
      error("The priority specified is invalid");
    if (mx_command && !entry_priority)
      error("A priority needs to specified for each MX record");
    if (arecord_command && !is_valid_ascii_ip(entry_target_string))
      error("The target IP is invalid");
    if (aaaa_command) 
      error("IPv6 Not yet supported.");
    if (ptr_command && !is_valid_domain(entry_target_string))
      error("The target domain is invalid");
    if (add_command && (!entry_target_string || !entry_string))
      error("You must specify an entry name and a target with an add command");
  }
  else {
    error("You must specify a command");
  }

  dbi_initialize(NULL);
  if ((conn = dbi_conn_new("mysql"))==NULL) {
    error("Unable to connect to MySQL DB");
  }

  dbi_conn_set_option(conn, "host", db_host);
  dbi_conn_set_option(conn, "username", db_user);
  dbi_conn_set_option(conn, "password", db_pass);
  dbi_conn_set_option(conn, "dbname", db_name);
  if (dbi_conn_connect(conn)==-1) {
    dbi_conn_error(conn, error_msg);
    error((char *)*error_msg);
  }

  result = dbi_conn_queryf(conn, "SELECT do.domain_id AS domain_id "
			   "FROM users AS u, "
			   "domains AS d, " 
			   "domainownerships AS do "
			   "WHERE u.name='%s' "
			   "AND u.id=do.user_id "
			   "AND do.domain_id=d.id "
			   "AND d.name='%s'", 
			   this_user, escape_sql(domain_name));
  if (dbi_result_next_row(result)) {
    domain_id = (int)dbi_result_get_long(result,"domain_id");
  } else {
    error("The domain you specified, does not appear to be yours");
  }
  result=NULL;

  /* Prompt user for confirmation */
  if (!batch_mode) {
    printf("Do you really want to go ahead with this [y/N]: ");
    if (getc(stdin)!='y') error("User requested abort");
  }

  /* If this was a list_command, get the records for the domain, and print
   * them to stdout */
  if (list_command) {
    result = dbi_conn_queryf(conn, "SELECT name,type,content,ttl,prio "
			     "FROM records "
			     "WHERE domain_id=%d",
			     domain_id);
    printf("; There are %d records\n",dbi_result_get_numrows(result));

    /* Do it this way, to get the first row as well */
    while (dbi_result_next_row(result)) {
      name = dbi_result_get_string(result, "name");
      type = dbi_result_get_string(result, "type");
      content = dbi_result_get_string(result, "content");
      prio = (int)dbi_result_get_long(result, "prio");
      ttl = (int)dbi_result_get_long(result, "ttl");
      printf("%-25s IN %6s %-30s %3d %6d\n",name,type,content,prio,ttl);
    }
    printf("; End of domain\n");

  } else {
    
    /* Determine update to make */
    if (arecord_command) {
      entry_type="A";
    } else if (cname_command) {
      entry_type="CNAME";
    } else if (mx_command) {
      entry_type="MX";
    } else if (ns_command) {
      entry_type="NS";
    } else if (aaaa_command) {
      entry_type="AAAA";
    } else if (ptr_command) {
      entry_type="PTR";
    } else {
      error("No entry type specified");
    }
    
    tp = (struct timeb*)malloc(sizeof(struct timeb));
    ftime(tp);
    
    if (add_command) {
      result = dbi_conn_queryf(conn, "INSERT INTO records "
			       "(domain_id,name,type,content,ttl,prio,change_date) "
			       "VALUES "
			       "(%i,'%s','%s','%s',%i,%i,%i)",
			       domain_id,
			       escape_sql(entry_string),
			       entry_type, 
			       escape_sql(entry_target_string),
			       entry_ttl,
			       entry_priority,
			       tp->time);
      if (result==NULL) {
	dbi_conn_error(conn, error_msg);
	error((char *)*error_msg);
      }
      increment_serial(domain_name,1,&conn);
    } else if (rem_command) {
      result = dbi_conn_queryf(conn, "DELETE FROM records "
			       "WHERE domain_id='%i' "
			       "AND name='%s' "
			       "AND type='%s' "
			       "AND content='%s' ",
			       domain_id,
			       escape_sql(entry_string),
			       entry_type,
			       escape_sql(entry_target_string));
      if (result==NULL) 
	error("An error occurred while removing entry from DB");
      
      increment_serial(domain_name,1,&conn);
      printf("%d entries were deleted\n",
	     (int)dbi_result_get_numrows_affected(result));
    } else {
      error("This should never happen 01");
    }
    result=NULL;
    
    printf("Your update was successfully executed\n");
  }
  return 0;
}
