/* $Id: zonemanipfunc.c,v 1.11 2003/12/28 00:20:13 boll Exp $ */
#include "zonemanipfunc.h"

void error(char* c) {
  printf("*** ");
  printf("%s\n",c);
  exit(1);
}

void usage() {
  printf("usage: zonemanip [options] domain\n"
	 "\n"
	 " -a, --add             Add an entry to the domain\n"
	 " -r, --remove          Remove an entry from the zone\n"
	 " -A, --a-record        Manipulate only A records\n"
	 " -C, --cname           Manipulate only CNAME records\n"
	 " -M, --mx              Manipulate only MX records\n");
  printf(" -N, --ns              Manipulate only NS records\n"
	 " -e, --entry=STRING    Name of the entry to manipulate\n"
	 " -t, --target=STRING   Target of the entry, if adding a new one\n"
	 " -p, --priority=INT    Priority of entry, if applicable\n"
	 " -l, --list            List contents of domain\n"
	 " -h, --help            Show this help\n"
	 " -v, --version         Show version information\n\n");
  printf(" -q, --yes             Assume yes on questions (batch mode)");
  printf("Please bear in mind that inputvalidation is not \n"
	 "complete, which means that you may be able to sneak\n"
	 "erroneous records past this program, causing havoc\n"
	 "with your domain.\n"
	 "\n"
	 "Should this happen please contact %s\n",MAINTAINER);
  exit(1);
}

void version() {
  printf("zonemanip version %s\n", VERSION);
  exit(1);
}

void get_pdns_mysql_db_credentials(char** pdns_conf,char** user,char** pass,char** db,char** host) {
  int fd;
  char c;
  int bufidx = 0;
  char* strbuf = (char *)malloc(100*sizeof(char));

  if ((fd = open(*pdns_conf,O_RDONLY,0))==-1) 
    error("Unable to open pdns config file");
  while (read(fd,&c,1)>0) {
    if (c=='\n') {
      strbuf[bufidx]='\0';
      bufidx=0;
      if (DEBUG>10) printf("Found: %s",strbuf);
      /* Check to see if this line is what we are looking for */
      if (strncmp("gmysql-host",strbuf,11)==0) {
	*host=get_value(strbuf);
	if (DEBUG) printf("Found mysql-host: '%s'\n",*host);
      } else if (strncmp("gmysql-user",strbuf,11)==0) {
	*user=get_value(strbuf);
	if (DEBUG) printf("Found mysql-user: '%s'\n",*user);
      } else if (strncmp("gmysql-dbname",strbuf,13)==0) {
	*db=get_value(strbuf);
	if (DEBUG) printf("Found mysql-db: '%s'\n",*db);
      } else if (strncmp("gmysql-password",strbuf,15)==0) {
	*pass=get_value(strbuf);
	if (DEBUG) printf("Found mysql-pass: '%s'\n",*pass);
      }
    } else {
      strbuf[bufidx++]=c;
    }
  }
  close(fd);
}

/* Takes a key=value pair, copies the value part
 * to a new char* and returns it, null-terminated.
 * It does not modify the original string */
char* get_value(char* c) {
  char state=0;
  int i=0;
  int copied=0;
  char* buffer;
  int buffer_size=50;

  buffer = (char *)malloc(buffer_size*sizeof(char));
  for (i=0;i<strlen(c);i++) {
    if (state==0 && c[i]=='=') {
      /* Next char begins the value */
      state=1;
    } else if (state==1) {
      /* This is the first char in the value part,
       * so copy it, and remember how much was copied */
      if (copied==buffer_size) {
	error("Buffer overflow in get_value()");
      }
      buffer[copied++]=c[i];
    } else {
      /* This is the key, skip it */
    }
  }
  buffer[copied]='\0';
  if (DEBUG) printf("Value: %s\n",buffer);
  return buffer;
}

int is_valid_domain(char* domain) {
  int i;
  for (i=0;i<strlen(domain);i++) {
    if (!is_valid_domain_char(domain[i])) {
      return 0;
    }
  }
  return 1;
}

int is_valid_domain_char(char c) {
  if ((c>=65 && c<=90) ||   /* A to Z */
      (c>=97 && c<=122) ||  /* a to z */
      (c>=48 && c<=57) ||   /* 0 to 9 */
      (c==45) || (c==46) || (c==42)) { /* dot, star and dash */
    return 1;
  }
  return 0;
}

int is_valid_ascii_ip(char* ip) {
  int i;
  if ((strlen(ip)<7) || (strlen(ip)>15))
    return 0;
  for (i=0;i<strlen(ip);i++) {
    if (!is_valid_ip_char(ip[i]))
      return 0;
  }
  return 1;
}

int is_valid_ip_char(char c) {
  if ((c==46) || (c>=48 && c<=57))
    return 1;
  return 0;
}

/* Increments the serial number for the specified
 * domain by the increase units */
int increment_serial(char* domain, int increase, dbi_conn *conn_ptr) {
  dbi_result result;
  char *serial, *soa_cpy, *master, *contact;
  int serial_int, domain_id;
  const char *soa;
  const char *delim = " ";
  result = dbi_conn_queryf(*conn_ptr, "SELECT r.content AS content, "
			   "d.id AS id "
			   "FROM records r, domains d "
			   "WHERE d.name='%s' "
			   "AND d.id=r.domain_id "
			   "AND r.type='SOA'",
			   domain);
  if (dbi_result_next_row(result)) {
    soa=dbi_result_get_string(result, "content");
    domain_id=dbi_result_get_long(result, "id");
    soa_cpy=(char*)malloc((strlen(soa)+1)*sizeof(char));
    strcpy(soa_cpy,soa);
    master=strtok(soa_cpy,delim);
    contact=strtok(NULL,delim);
    serial=strtok(NULL,delim);
    if (DEBUG) {
      printf("master=%s contact=%s serial=%s\n",master,contact,serial);
    }
    serial_int=atoi(serial);
    serial_int+=increase;
    result = dbi_conn_queryf(*conn_ptr, "UPDATE records "
			     "SET content='%s %s %i' "
			     "WHERE domain_id=%i "
			     "AND type='SOA'",
			     master,
			     contact,
			     serial_int,
			     domain_id);
    if (result==NULL) {
      error("Unable to increment serial");
    }
    return 1;
  } else {
    error("The domain does not appear to have a SOA record. This is bad.");
    return 0;
  }
  
}

/* Returns true if the given domain-name has
 * a serial of 0, which is required in powerdns
 * to notice changes to records */
int serial_is_zero(char* domain, dbi_conn *conn_ptr) {
  dbi_result result;
  char *serial, *soa_cpy;
  const char *soa;
  const char *delim = " ";

  result = dbi_conn_queryf(*conn_ptr, "SELECT r.content AS content "
			   "FROM records r, domains d "
			   "WHERE d.name='%s' "
			   "AND d.id=r.domain_id "
			   "AND r.type='SOA'",
			   domain);
  if (dbi_result_next_row(result)) {
    soa=dbi_result_get_string(result, "content");
    soa_cpy=(char*)malloc((strlen(soa)+1)*sizeof(char));
    strcpy(soa_cpy,soa);
    strtok(soa_cpy,delim);
    strtok(NULL,delim);
    serial=strtok(NULL,delim);
    if (DEBUG) printf("Serial: %s (%c)\n",serial,serial[0]);
    if ((strlen(serial)==1) && serial[0]=='0') return 1;
    else return 0;
  } else {
    error("The domain does not appear to have a SOA record. This is bad.");
    return 0;
  }
}

/* Escape bad characters (in the sql sense), but do
 * not modify original string. */
char* escape_sql(char* sql) {
  int escape_count=0,i=0,j=0;
  char* new_sql;
  /* Count chars to be escaped */
  for (i=0;i<strlen(sql);i++) {
    if (sql[i]==';' || sql[i]=='\'') escape_count++;
  }
  /* Allocate a new string of suitable length */
  new_sql = (char*)malloc((strlen(sql)+escape_count+1)*sizeof(char));
  /* Copy original string to new space, making
   * sure to escape the relevant chars */
  for (i=0;i<strlen(sql);i++,j++) {
    if (sql[i]==';' || sql[i]=='\'') {
      new_sql[j]='\\';
      new_sql[++j]=sql[i];
    } else {
      new_sql[j]=sql[i];
    }
  }
  new_sql[j]='\0';
  /* Return new string */
  return new_sql;
}
